<h3><a href="http://pypi.python.org/pypi/nose/">Download</a></h3>
<ul>
  <li><a href="http://pypi.python.org/pypi/nose/{{ release }}">
      Current version: {{ release }}</a>
  </li>
</ul>

<h3>Install</h3>
<ul>
  <li>This release:<br/>
    <tt>easy_install nose=={{ release }}</tt></li>
  <li>Development (unstable):<br/>
    <tt>easy_install nose==dev</tt>
  </li>
</ul>

<h3>Community</h3>
<ul>
  <li><a href="http://groups.google.com/group/nose-users">
    Users' discussion list</a>
    <ul><li>Talk about using nose. Get help. Give help!</li></ul>
  </li>
  <li><a href="http://lists.idyll.org/listinfo/testing-in-python">
      TIP list</a>
    <ul><li>The Testing In Python list features wide-ranging
        discussions of all topics of interest to python
        testers.</li></ul>
  </li>
</ul>

<h3><a href="https://github.com/nose-devs/nose/">Tracker</a></h3>
<ul><li>Report bugs, request features, wik the wiki, browse source.</li></ul>

<h3>Other links</h3>
<ul>
  <li>
    <a href="http://pytest.org">pytest</a>
  </li>
  <li>
    <a href="http://peak.telecommunity.com/DevCenter/setuptools">
      setuptools</a>
  </li>
</ul>

